local quad = {}

local function load_quad()
    for x=0, 64 - 1 do
        quad[x] = love.graphics.newQuad(x,0,1,64,64,64)
    end
end

load_quad()

local function draw_floor_ceil()
    oggee.shader.update_floor_ceil_values()
    love.graphics.setShader(oggee.shader.floor_ceil)
    love.graphics.rectangle("fill", 0,0,oggee.screen.w, oggee.screen.h)
    love.graphics.setShader()
end

local function draw_walls()
    local buffer = oggee.draw.buffer
    buffer = {}
    local camera = oggee.camera
    for x = 0, oggee.screen.w - 1 do
        local angle = math.rad((camera.fov / (oggee.screen.w - 1))) * x - math.rad(camera.fov/2) + camera.dir
        local dist, ix, iy, ver, id = oggee.dda(camera.x, camera.y, angle, 64, oggee.map.size.w, oggee.map.size.h, oggee.map.wall, 20)
        if dist then
            buffer[x] = dist
            local tx = 0
            local a = math.cos(angle)
            local b = math.sin(angle)
            if ver then
                tx = math.floor(ix % 64)
                if math.deg(angle) % 360 < 180 then
                    tx = 63 - tx
                end
            else
                tx = math.floor(iy % 64)
                local na = math.deg(angle) % 360
                if na > 90 and na < 270 then
                    tx = 63 - tx
                end
            end
            dist = dist * math.cos(angle - camera.dir)
            local h1 = camera.pdist / dist * camera.z
            local h2 = camera.pdist / dist * (64-camera.z)
            local scale = (h1+h2) / 64
            love.graphics.draw(oggee.texture.textures[id][1], quad[tx], x, oggee.screen.h / 2 + h1 - (h1+h2), 0, 1, scale)
        else
            buffer[x] = 9999
        end
        -- local x1, y1, x2, y2 = camera.x , camera.y, camera.x + math.cos(angle) * 100, camera.y + math.sin(angle) * 100
        -- love.graphics.line(x1,y1,x2,y2)
    end
    oggee.draw.buffer = buffer
end

local function draw_3d_world()
    love.graphics.scale(oggee.screen.sx, oggee.screen.sy)
    draw_floor_ceil()
    draw_walls()
    oggee.shader.update_sprite_shader()
    oggee.sprite.draw()
end


local function debug()
    local mapX = oggee.map.size.w
    local mapY = oggee.map.size.h
    for x=0, mapX - 1 do
        for y=0, mapY - 1 do
            local id = y * mapX + x + 1
            if oggee.map.wall[id] > 0 then
                love.graphics.rectangle("fill", x * 64, y * 64, 64, 64)
            end
        end
    end
end

local draw = {
    world = draw_3d_world,
    debug = debug,
    buffer = {}
}

return draw