local floor_ceil = [[
    extern vec4 camera;
    extern vec2 screen;
    extern float pdist;
    extern float height;
    extern Image floor;
    extern Image ceil;
    extern int mapX;
    extern int mapY;
    extern int sx;
    extern int sy;

    vec4 effect( vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords ){
        int maxdist = 400;
        screen_coords = screen_coords / vec2(sx,sy) * vec2(screen.x,screen.y);

        float z = 0.0;
        bool isfloor = true;

        if (screen_coords.y >= screen.y / 2)  {
            z = height / (screen.y / 2 - (screen.y-(screen_coords.y))) * pdist;
        }
        if (screen_coords.y < screen.y / 2)  {
            isfloor = false;
            z = (64 - height) / (screen.y / 2 - ((screen_coords.y))) * pdist;
        }
        float angle = radians(camera.w / (screen.x - 1))*(screen_coords.x) - radians(camera.w/2) + camera.z;
        z = z / cos(angle - camera.z);
    
        float wx = cos(angle) * z + camera.x;
        float wy = sin(angle) * z + camera.y;

        float shader = 1;


        if (wx / 64 >= 0 && wx / 64 < mapX && wy / 64 >= 0 && wy / 64 < mapY) {
            float tx = mod(wx, mapX * 64) / (mapX * 64);
            float ty = mod(wy, mapY * 64) / (mapY * 64);
            vec4 pixel;
            if (isfloor == true){
                pixel = Texel(floor, vec2(tx,ty));
            }
            else{
                pixel = Texel(ceil, vec2(tx, ty));
            }
            return pixel * shader;
        }
        return vec4(0,0,0,0);
    }
]]

local sprite = [[
    extern Image tex;
    extern Image buffer;
    extern float depth;
    extern vec2 scale;
    extern vec2 screen;
    vec4 effect( vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords ) {
        
        vec4 value = Texel(buffer, vec2(screen_coords.x / screen.x / scale.x,1));

        if (depth <= value.r){
            vec4 pixel = Texel(tex, texture_coords);
            return pixel;
        }
        return vec4(0,0,0,0);
    }
]]

function update_floor_ceil_values()
    local camera = oggee.camera

    local floor_ceil = oggee.shader.floor_ceil

    floor_ceil:send("camera", {camera.x, camera.y, camera.dir, camera.fov})
    floor_ceil:send("height", camera.z)
end



local function update_all_floor_ceil_values()
    local camera = oggee.camera

    local floor_ceil = oggee.shader.floor_ceil

    floor_ceil:send("camera", {camera.x, camera.y, camera.dir, camera.fov})
    floor_ceil:send("screen", {oggee.screen.w, oggee.screen.h})
    floor_ceil:send("pdist", camera.pdist)
    floor_ceil:send("height", camera.z)
    floor_ceil:send("floor", oggee.texture.floor_img)
    floor_ceil:send("ceil", oggee.texture.ceil_img)
    floor_ceil:send("mapX", oggee.map.size.w)
    floor_ceil:send("mapY", oggee.map.size.h)
    floor_ceil:send("sx", oggee.screen.sx * oggee.screen.w)
    floor_ceil:send("sy", oggee.screen.sy * oggee.screen.h)
end

function replace(x)
    local value = oggee.draw.buffer[x]
    return value,0,0,1
end

local wall_img_data = love.image.newImageData(oggee.screen.w,1, "rg32f")

function update_sprite_buffer_img()
    wall_img_data = love.image.newImageData(oggee.screen.w,1, "rg32f")
end

local function update_sprite_shader()
    local sprite_shader = oggee.shader.sprite
    
    wall_img_data:mapPixel(replace,0,0,oggee.screen.w,1)
    sprite_shader:send("buffer", love.graphics.newImage(wall_img_data))
end

local shader = {
    floor_ceil = love.graphics.newShader(floor_ceil),
    sprite = love.graphics.newShader(sprite),
    update_all_floor_ceil_values = update_all_floor_ceil_values,
    update_floor_ceil_values = update_floor_ceil_values,
    update_sprite_buffer_img = update_sprite_buffer_img,
    update_sprite_shader = update_sprite_shader
}

return shader