local function create(x, y, z, img, i, visible)
    if visible == nil then
        visible = true
    end
    oggee.sprite.sprites[i] = {x, y, z, img, visible} 
end

local function set_image(img, i)
    oggee.sprite.sprites[i][4] = img
end

local function set_visibility(visible, i)
    oggee.sprite.sprites[i][4] = visible
end

local function set_position(x,y,z,i)
    oggee.sprite.sprites[i][1] = x
    oggee.sprite.sprites[i][2] = y
    oggee.sprite.sprites[i][3] = z
end

local function removekey(table, key)
    local element = table[key]
    table[key] = nil
    return element
end

local function remove_sprite(i)
    oggee.sprite.sprites[i] = nil
end

local function sprite_exists(i)
    return oggee.sprite.sprites[i]
end


function ab(a, b)
    local camera = oggee.camera
    local dist1 = math.sqrt((a[1] - camera.x)^2+(a[2] - camera.y)^2)
    local dist2 = math.sqrt((b[1] - camera.x)^2+(b[2] - camera.y)^2)
    return dist1 > dist2
end

function deepcopy(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in next, orig, nil do
            copy[deepcopy(orig_key)] = deepcopy(orig_value)
        end
        setmetatable(copy, deepcopy(getmetatable(orig)))
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end


local function draw_sprite()
    local sprites = oggee.sprite.sprites
    local s = {}
    local f = 0
    for i,v in pairs(sprites) do
        f = f +1
        s[f] = {v[1],v[2],v[3],v[4]}
    end
    table.sort(s, ab)
    local camera = oggee.camera
    local pdist = camera.pdist
    local screen_w =oggee.screen.w
    local screen_h =oggee.screen.h
    local sprite_shader = oggee.shader.sprite
    for i,v in pairs(s) do
        local _sx = v[1] - camera.x
        local _sy = v[2] - camera.y
        local sz = v[3] - (32-camera.z)
 
        local cs = math.cos(camera.dir)
        local sn = math.sin(camera.dir) 
    
        local a = _sy * cs + _sx * -sn
        local b = _sx * cs - _sy * -sn 

        _sx = a
        _sy = b

        _sx = (_sx*pdist/_sy)+(screen_w/2)
        _sy = (sz*pdist/_sy)+(screen_h/2)
    
        local scale = pdist / b * 64

        if b > 0 then
            love.graphics.setShader(sprite_shader)
        
            sprite_shader:send("tex", v[4])
            sprite_shader:send("depth", b)
            sprite_shader:send("scale", {oggee.screen.sx,oggee.screen.sy})
            sprite_shader:send("screen", {oggee.screen.w,oggee.screen.h})
            love.graphics.draw(v[4], _sx - scale/2, _sy - scale/2, 0, scale/64, scale/64)
        end
    end
    love.graphics.setShader()
end


local sprite = {
    create = create,
    draw = draw_sprite,
    sprites = {},
    set_image = set_image,
    set_visibility = set_visibility,
    setPosition = set_position,
    remove = remove_sprite,
    spriteExists = sprite_exists,
}

return sprite